@@ -7,6 +7,11 @@ from telliot_core.datafeed import DataFeed
 from telliot_core.utils.response import ResponseStatus
 from web3.datastructures import AttributeDict
 
+from telliot_feed_examples.utils.log import get_logger
+
+
+logger = get_logger(__name__)
+
 
 async def tip_query(
     oracle: Contract,
@@ -21,6 +26,7 @@ async def tip_query(
     reporters to report relevant data."""
     tx_receit, status = await oracle.write_with_retry(
         func_name="tipQuery",
+        gas_limit=350000,
         gas_price=gas_price,
         extra_gas_price=20,
         retries=retries,
